// The MIT License
//
// Copyright (c) 2004 Evren Sirin
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.

/*
 * Created on Dec 27, 2003
 *
 */
package org.mindswap.owls.profile;

import java.net.URI;
import java.util.List;

import org.mindswap.owl.OWLDataValueList;
import org.mindswap.owl.OWLIndividual;
import org.mindswap.owl.OWLIndividualList;
import org.mindswap.owl.OWLObjectProperty;
import org.mindswap.owls.process.Input;
import org.mindswap.owls.process.InputList;
import org.mindswap.owls.process.MultiConditional;
import org.mindswap.owls.process.Output;
import org.mindswap.owls.process.OutputList;
import org.mindswap.owls.process.Process;
import org.mindswap.owls.process.Result;
import org.mindswap.owls.process.ResultList;
import org.mindswap.owls.service.Service;
import org.mindswap.owls.vocabulary.OWLS;

/**
 * Represents the OWL-S profile.
 * 
 * OWL-S concept: http://www.daml.org/services/owl-s/1.0/Profile.owl#Profile
 *  
 * @author Evren Sirin
 *
 */
public interface Profile extends OWLIndividual, MultiConditional {	
	/**
	 * Get the service this profile presents
	 * 
	 * @return
	 */
	public Service getService();
	
	/**
	 * Set the service this profile presents
	 * 
	 * @param service
	 */
	public void setService(Service service);

	/**
	 * Get the process associated with this profile. This process is the one that is the 
	 * value of profile:has_process parameter
	 * 
	 * @return
	 */
	public Process getProcess();

	/**
	 * Set profile's text description. No language identifier will be used.
	 * 
	 * @param desc
	 */
	public void setTextDescription(String desc);
	
	/**
	 * Get profile's text description. See {@link org.mindswap.owl.OWLConfig#DEFAULT_LANGS OWLConfig}
	 * to learn how the language identifiers will be resolved when searching for a text description. 
	 * 
	 * @return
	 */
	public String getTextDescription();
	
	/**
	 * Get profile's text description. The associated textDescription should have the same language 
	 * identifier as given in the parameter. If a textDescription for that labguage is not found null
	 * value will be returned even if there is another textDescription with a different language 
	 * identifier. 
	 * 
	 * @param lang
	 * @return
	 */
	public String getTextDescription(String lang);
	
	/**
	 * Return all text descriptions written in any language.
	 * 
	 * @return
	 */
	public OWLDataValueList getTextDescriptions();
	
	/**
	 * Get the service name defined in the profile. See {@link org.mindswap.owl.OWLConfig#DEFAULT_LANGS OWLConfig}
	 * to learn how the language identifiers will be resolved when searching for a name. 
	 * 
	 * @return
	 */
	public String getServiceName();
	
	/**
	 * Get the service name defined in the profile. The associated serviceName should have the same 
	 * language identifier as given in the parameter. If a serviceName for that language is not found 
	 * null value will be returned even if there is another serviceName with a different language 
	 * identifier. 
	 * 
	 * @param lang
	 * @return
	 */
	public String getServiceName(String lang);
	
	/**
	 * Return all service names written in any language.
	 * 
	 * @return
	 */
	public OWLDataValueList getServiceNames();
	
	/**
	 * Set the service name. No language identifier will be used.
	 * 
	 * @param desc
	 */
	public void setServiceName(String desc);
	
	public void addInput(Input input);
	
	public void addOutput(Output output);
	
	public void addResult(Result result);
	
	public void setResult(Result result);	
		
	/**
	 * Get the inputs for this profile.
	 * 
	 * @return
	 */
	public InputList getInputs();
	
	/**
	 * Get the outputs for this profile.
	 * 
	 * @return
	 */
	public OutputList getOutputs();
	
	/**
	 * Get the results for this profile.
	 * 
	 * @return
	 */
	public ResultList getResults();		
	
	/**
	 * Get the result for this profile. In case there are multiple results defined for this
	 * profile a random one is returned
	 * 
	 * @return
	 */
	public Result getResult();	
	
	/**
	 * getCategory
	 * 
	 * @return
	 */
	public ServiceCategory getCategory();
	
	/**
	 * Get the list of all categories defined for this profile.
	 * 
	 * @return
	 */
	public OWLIndividualList getCategories();
	
	/**
	 * Add a new category for this profile.
	 * 
	 * @param category
	 */
	public void addCategory(ServiceCategory category);

	/**
	 * Set the category for this profile (remove any previous cateogry)
	 * @param category
	 */
	public void setCategory(ServiceCategory category);
	
	/**
	 * getContactInfo
	 * 
	 * @return
	 */
	public Actor getContactInfo();

	/**
	 * Get all the contact info values
	 * 
	 * @return
	 */
	public OWLIndividualList getContactInfos();
	
	public void addContactInfo(Actor actor);

	public void setContactInfo(Actor actor);
	
	/**
	 * getServiceParameters
	 * 
	 * @return
	 */
	public OWLIndividualList getServiceParameters();
	
	/**
	 * getServiceParameter
	 * 
	 * @return
	 */
	public ServiceParameter getServiceParameter(OWLObjectProperty prop);	
	
	public ServiceParameter getServiceParameter(String name);
//---- Ajout par faten ----------------------------------
	//public ServiceParameter getServiceParameter();
//-------------------------------------------------------
	
	public OWLIndividual getServiceParameterValue(OWLObjectProperty prop);
	
	public OWLIndividual getServiceParameterValue(String name);
	
	public void addServiceParameter(ServiceParameter serviceParam);	
	
	public ServiceParameter createServiceParameter( OWLObjectProperty prop, OWLIndividual sParameter );

	public ServiceParameter createServiceParameter( String name, OWLIndividual sParameter );
	
	/**
	 * @deprecated Use createServiceParameter(OWLObjectProperty, OWLIndividual) instead.
	 */
	public void addServiceParameterValue(OWLObjectProperty prop, OWLIndividual value);
	
	/**
	 * @deprecated Use createServiceParameter(String, OWLIndividual) instead.
	 */
	public void addServiceParameterValue(String name, OWLIndividual value);
/*  *************************************** Ajout par Faten ***********************************	*/
/*	public LinguisticPhenomenon getLinguisticPhenomenon(OWLObjectProperty prop);	
	
	public LinguisticPhenomenon getLinguisticPhenomenon(String name);
	
	public OWLIndividual getLinguisticPhenomenonValue(OWLObjectProperty prop);
	
	public OWLIndividual getLinguisticPhenomenonValue(String name);
	
	public void addLinguisticPhenomenon(LinguisticPhenomenon LinguisticPheno);	
	
	public LinguisticPhenomenon createLinguisticPhenomenon( OWLObjectProperty prop, OWLIndividual sParameter );

	public LinguisticPhenomenon createLinguisticPhenomenon( String name, OWLIndividual sParameter );
	
	public void addLinguisticPhenomenonValue(OWLObjectProperty prop, OWLIndividual value);

	public void addLinguisticPhenomenonValue(String name, OWLIndividual value);
*/
/*	*********************************************************************************************************	*/	
	public URI getServiceProduct();
	public OWLDataValueList getServiceProducts();
	public void addServiceProduct(URI product);
	public void setServiceProduct(URI product);
	
	public URI getServiceClassification();
	public OWLDataValueList getServiceClassifications();
	public void addServiceClassification(URI classification);
	public void setServiceClassification(URI classification);
	
//--------------------------------------- Faten -----------------------------------------
	//public String getLinguisticPhenomenon();  /*	****************Supp par Faten : Le 23/07/2012  ********* */
	public String getDescription();  //J'ai remplac: getLinguisticPhenomenon() par: getDescription()car elle va donner TextgetLinguisticPhenomenon  ----> getDescription
//--------------------------------------- Faten -----------------------------------------	
	//public LinguisticPhenomenon getPhenomenon();  /*	****************Supp par Faten : Le 23/07/2012  ********* */
	public void setLinguisticPhenomenon(String linguisticPhenomenon11);
	public LinguisticPhenomenon getLinguisticPhenomenon(OWLObjectProperty prop);
//------------------------------------- 	Faten : Le 23/07/2012  --------------------------------------
	public void addLinguisticPhenomenon(LinguisticPhenomenon phenomenon);
	public LinguisticPhenomenon getLinguisticPhenomenon();

//------------------------------------------Faten ---------- Le 21/07/2012 --------------------------------------
	//------------------------------ Je vais pas considerer la relation d'hritage  -----------------------------
	public ServiceProcessing_Level getServiceProcessing_Level();
	public ServiceProcessing_Level getServiceProcessing_Level(OWLObjectProperty prop);
	public void setServiceProcessing_Level(String ServiceProcessing_Level);
	public void addServiceProcessing_Level(ServiceProcessing_Level servicePL);
//------------------------------------- 	Faten : Le 23/07/2012  --------------------------------------
	public Resource_Ling getResource();
	public Resource_Ling getResource(OWLObjectProperty prop);
	public void setResource(String resource);
	public void addResource(Resource_Ling resource);

	public Approach getApproach();
	public Approach getApproach(OWLObjectProperty prop);
	public void setApproach(String approach);
	//public void addApproach(Approach approach); //@@@@@@@@@@@ supp par Faten Le 27/07/2012  @@@@@@@@@@@

	
	
	public Analysis_Type getAnalysis_Type();
	public Analysis_Type getAnalysis_Type(OWLObjectProperty prop);
	public void setAnalysis_Type(String analysis_Type);
	public void addAnalysis_Type(Analysis_Type analysis_Type);
//-------------------------- Faten:  Le 24/07/2012 -------------------- pour l'extraction dans la classe: Services
	public LinguisticFormalism getLinguisticFormalism();
	public LinguisticFormalism getLinguisticFormalism(OWLObjectProperty prop);
	public void setLinguisticFormalism(String analysis_Type);
	public void addLinguisticFormalism(LinguisticFormalism linguisticFormalism);
}										